#ifndef EXPERIMENT_H_
#define EXPERIMENT_H_
// ********************************************************************
// * The BornAgain project                                            *
// * Simulation of neutron and x-ray scattering at grazing incidence  *
// *                                                                  *
// * LICENSE AND DISCLAIMER                                           *
// * Lorem ipsum dolor sit amet, consectetur adipiscing elit.  Mauris *
// * eget quam orci. Quisque  porta  varius  dui,  quis  posuere nibh *
// * mollis quis. Mauris commodo rhoncus porttitor.                   *
// ********************************************************************
//! @file   Experiment.h
//! @brief  Definition of Experiment class
//! @author Scientific Computing Group at FRM II
//! @date   20.06.2012

#include "ISample.h"
#include "OutputData.h"
#include "Beam.h"
#include "Detector.h"

class Experiment
{
public:
	Experiment();
	Experiment(ISample *p_sample);
    virtual ~Experiment() {}

    /// Run a simulation with the current parameter settings
    virtual void runSimulation();

    /// Normalize the detector counts
    virtual void normalize();

    /// Set the sample to be tested
    void setSample(ISample *p_sample);

    /// get the sample
    ISample *getSample() { return mp_sample; }
    const ISample *getSample() const { return mp_sample; }

    /// Get data structure that contains the intensity map on the detector for all scan parameters
    OutputData<double>* getOutputDataClone() const;

    const OutputData<double>* getOutputData() const;

    Beam getBeam() const
    {
        return m_beam;
    }

    void setBeam(Beam beam)
    {
        m_beam = beam;
    }

    Detector getDetector() const
    {
        return m_detector;
    }

    void setBeamParameters(double lambda, double alpha_i, double phi_i);

    void setBeamIntensity(double intensity);

    void setOutputDataMask(size_t n_chunks_total=1, size_t n_chunk=0);

    const OutputData<double>* getOutputDataMask() const;

    //! create combined parameter pool of the whole sample and experiment (if any)
    virtual ParameterPool *createParameterTree() const;

protected:
    /// Default implementation only adds the detector axes
    virtual void updateIntensityMapAxes();

    ISample *mp_sample;
    Detector m_detector;
    Beam m_beam;
    OutputData<double> m_intensity_map;
    OutputData<double> m_current_output_data_mask;
    bool m_is_normalized;
};





#endif /* EXPERIMENT_H_ */
